==> dart_redux_overview-0.0.1.tar.gz: lib/src/analyzer.dart <==
import 'dart:io';

import 'package:analyzer/dart/analysis/utilities.dart';
import 'package:analyzer/dart/analysis/features.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';

import 'result.dart';

class ClassesAnalyzer {
  Future<Analysis> analyze(List<File> files) async {
    final classes = <ClassAnalysis>[];

    for (var file in files) {
      final content = await file.readAsString();
      var result = parseString(
        content: content,
        path: file.path,
        featureSet: FeatureSet.fromEnableFlags([]),
      );

      classes.addAll(_generateFile(result.unit));
    }

    return Analysis(
      classes: classes,
    );
  }

  List<ClassAnalysis> _generateFile(CompilationUnit unit) {
    final classes = <ClassAnalysis>[];
    final visitor = _ClassVisitor(classes);
    unit.visitChildren(visitor);
    return classes;
  }
}

class _ClassVisitor extends SimpleAstVisitor {
  final List<ClassAnalysis> classes;

  _ClassVisitor(this.classes);

  @override
  dynamic visitClassDeclaration(ClassDeclaration node) {
    final annotations = _AnnotationVisitor();
    node.visitChildren(annotations);
    if (annotations.annotations.any((a) => a.name.name == 'freezed')) {
      print('Analyzing class : ${node.name}');
      final constructors = _ConstructorVisitor();
      node.visitChildren(constructors);
      classes.add(
        ClassAnalysis(
          name: node.name.name,
          constructors: constructors.constructors,
        ),
      );
    }
  }
}

class _AnnotationVisitor extends SimpleAstVisitor {
  final List<Annotation> annotations = [];

  _AnnotationVisitor();

  @override
  dynamic visitAnnotation(Annotation node) {
    annotations.add(node);
    return super.visitAnnotation(node);
  }
}

class _ConstructorVisitor extends SimpleAstVisitor {
  final List<ConstructorAnalysis> constructors = [];

  String _formatDocumentation(Comment comment) {
    if (comment != null &&
        comment.tokens != null &&
        comment.tokens.isNotEmpty) {
      return comment.tokens.first.toString().replaceFirst('///', '').trim();
    }
    return null;
  }

  @override
  dynamic visitConstructorDeclaration(ConstructorDeclaration node) {
    constructors.add(
      ConstructorAnalysis(
        name: node.name?.name,
        comments: _formatDocumentation(node.documentationComment),
        requiredParameters: [
          ...node.parameters.parameters
              .where((x) => x.isRequiredPositional)
              .whereType<SimpleFormalParameter>()
              .map(
                (e) => ConstructorRequiredParameterAnalysis(
                  name: e.identifier.name,
                  type: e.type?.toString(),
                ),
              ),
        ],
        namedParameters: [
          ...node.parameters.parameters
              .where((x) => x.isNamed)
              .whereType<DefaultFormalParameter>()
              .map(
                (e) => ConstructorNamedParameterAnalysis(
                  name: e.identifier.name,
                  type:
                      (e.parameter as SimpleFormalParameter)?.type?.toString(),
                ),
              ),
        ],
      ),
    );
  }
}

==> dart_redux_overview-0.0.2.tar.gz: lib/src/analyzer.dart <==
import 'dart:io';

import 'package:analyzer/dart/analysis/utilities.dart';
import 'package:analyzer/dart/analysis/features.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';

import 'result.dart';

class ClassesAnalyzer {
  Future<Analysis> analyze(List<File> files) async {
    final classes = <ClassAnalysis>[];

    for (var file in files) {
      final content = await file.readAsString();
      var result = parseString(
        content: content,
        path: file.path,
        featureSet: FeatureSet.fromEnableFlags([]),
      );

      classes.addAll(_generateFile(result.unit));
    }

    return Analysis(
      classes: classes,
    );
  }

  List<ClassAnalysis> _generateFile(CompilationUnit unit) {
    final classes = <ClassAnalysis>[];
    final visitor = _ClassVisitor(classes);
    unit.visitChildren(visitor);
    return classes;
  }
}

class _ClassVisitor extends SimpleAstVisitor {
  final List<ClassAnalysis> classes;

  _ClassVisitor(this.classes);

  @override
  dynamic visitClassDeclaration(ClassDeclaration node) {
    final annotations = _AnnotationVisitor();
    node.visitChildren(annotations);
    if (annotations.annotations.any((a) => a.name.name == 'freezed')) {
      print('Analyzing "${node.name}" class ...');
      final constructors = _ConstructorVisitor();
      node.visitChildren(constructors);
      classes.add(
        ClassAnalysis(
          name: node.name.name,
          constructors: constructors.constructors,
        ),
      );
    }
  }
}

class _AnnotationVisitor extends SimpleAstVisitor {
  final List<Annotation> annotations = [];

  _AnnotationVisitor();

  @override
  dynamic visitAnnotation(Annotation node) {
    annotations.add(node);
    return super.visitAnnotation(node);
  }
}

class _ConstructorVisitor extends SimpleAstVisitor {
  final List<ConstructorAnalysis> constructors = [];
  final List<String> ignored;

  _ConstructorVisitor({
    this.ignored = const ['fromJson'],
  });

  String _formatDocumentation(Comment comment) {
    if (comment != null &&
        comment.tokens != null &&
        comment.tokens.isNotEmpty) {
      return comment.tokens.first.toString().replaceFirst('///', '').trim();
    }
    return null;
  }

  @override
  dynamic visitConstructorDeclaration(ConstructorDeclaration node) {
    if (!ignored.any((element) => node.name?.name == element)) {
      constructors.add(
        ConstructorAnalysis(
          name: node.name?.name,
          comments: _formatDocumentation(node.documentationComment),
          requiredParameters: [
            ...node.parameters.parameters
                .where((x) => x.isRequiredPositional)
                .whereType<SimpleFormalParameter>()
                .map(
                  (e) => ConstructorRequiredParameterAnalysis(
                    name: e.identifier.name,
                    type: e.type?.toString(),
                  ),
                ),
          ],
          namedParameters: [
            ...node.parameters.parameters
                .where((x) => x.isNamed)
                .whereType<DefaultFormalParameter>()
                .map(
                  (e) => ConstructorNamedParameterAnalysis(
                    name: e.identifier.name,
                    type: (e.parameter as SimpleFormalParameter)
                        ?.type
                        ?.toString(),
                  ),
                ),
          ],
        ),
      );
    }
  }
}
